@@ -27,6 +27,32 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: args.h,v $
+ * Revision 1.28  2005/11/30 12:47:37  csoutheren
+ * Removed tabs, reformatted some code, and changed tags for Doxygen
+ *
+ * Revision 1.27  2005/11/25 03:43:47  csoutheren
+ * Fixed function argument comments to be compatible with Doxygen
+ *
+ * Revision 1.26  2005/11/20 22:02:59  dereksmithies
+ * Fix tags so they work with doxygen.
+ *
+ * Revision 1.25  2005/01/26 05:37:43  csoutheren
+ * Added ability to remove config file support
+ *
+ * Revision 1.24  2003/09/17 01:18:01  csoutheren
+ * Removed recursive include file system and removed all references
+ * to deprecated coooperative threading support
+ *
+ * Revision 1.23  2003/03/27 07:27:07  robertj
+ * Added function to get a bunch of arguments as a string array.
+ *
+ * Revision 1.22  2002/09/16 01:08:59  robertj
+ * Added #define so can select if #pragma interface/implementation is used on
+ *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
+ *
+ * Revision 1.21  2001/12/15 04:49:06  robertj
+ * Added stream I/O functions for argument list.
+ *
  * Revision 1.20  2000/05/25 11:05:31  robertj
  * Added PConfigArgs class so can save program arguments to config files.
  *
@@ -95,11 +121,10 @@
 #ifndef _PARGLIST
 #define _PARGLIST
 
-#ifdef __GNUC__
+#ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
-
 /** This class allows the parsing of a set of program arguments. This translates
    the standard argc/argv style variables passed into the main() function into
    a set of options (preceded by a '-' character) and parameters.
@@ -122,50 +147,51 @@ class PArgList : public PObject
        next argument.
      */
     PArgList(
-      const char * theArgPtr = NULL,   /// A string constituting the arguments
-      const char * argumentSpecPtr = NULL,
-      /** The specification C string for argument options. See description for
-         details.
-       */
-      BOOL optionsBeforeParams = TRUE  /// Parse options only before parameters
+      const char * theArgPtr = NULL,        ///< A string constituting the arguments 
+      const char * argumentSpecPtr = NULL,  ///< The specification C string for argument options. See description for details.
+      BOOL optionsBeforeParams = TRUE       ///< Parse options only before parameters 
     );
     /** Create an argument list. */
     PArgList(
-      const PString & theArgStr,   /// A string constituting the arguments
-      const char * argumentSpecPtr = NULL,
-      /** The specification C string for argument options. See description for
-         details.
-       */
-      BOOL optionsBeforeParams = TRUE  /// Parse options only before parameters
+      const PString & theArgStr,             ///< A string constituting the arguments 
+      const char * argumentSpecPtr = NULL,   ///< The specification C string for argument options. See description for details.
+      BOOL optionsBeforeParams = TRUE        ///< Parse options only before parameters 
     );
     /** Create an argument list. */
     PArgList(
-      const PString & theArgStr,   /// A string constituting the arguments
-      const PString & argumentSpecStr,
-      /** The specification string for argument options. See description for
-         details.
-       */
-      BOOL optionsBeforeParams = TRUE  /// Parse options only before parameters
+      const PString & theArgStr,             ///< A string constituting the arguments 
+      const PString & argumentSpecStr,       ///< The specification string for argument options. See description for details.
+      BOOL optionsBeforeParams = TRUE        ///< Parse options only before parameters 
     );
     /** Create an argument list. */
     PArgList(
-      int theArgc,     /// Count of argument strings in theArgv
-      char ** theArgv, /// An array of strings constituting the arguments
-      const char * argumentSpecPtr = NULL,
-      /** The specification C string for argument options. See description for
-         details.
-       */
-      BOOL optionsBeforeParams = TRUE  /// Parse options only before parameters
+      int theArgc,                           ///< Count of argument strings in theArgv 
+      char ** theArgv,                       ///< An array of strings constituting the arguments 
+      const char * argumentSpecPtr = NULL,   ///< The specification C string for argument options. See description for details.
+      BOOL optionsBeforeParams = TRUE        ///< Parse options only before parameters 
     );
     /** Create an argument list. */
     PArgList(
-      int theArgc,     /// Count of argument strings in theArgv
-      char ** theArgv, /// An array of strings constituting the arguments
-      const PString & argumentSpecStr,
-      /** The specification string for argument options. See description for
-         details.
-       */
-      BOOL optionsBeforeParams = TRUE  /// Parse options only before parameters
+      int theArgc,                           ///< Count of argument strings in theArgv 
+      char ** theArgv,                       ///< An array of strings constituting the arguments 
+      const PString & argumentSpecStr,       ///< The specification string for argument options. See description for details.
+      BOOL optionsBeforeParams = TRUE        ///< Parse options only before parameters 
+    );
+  //@}
+
+  /**@name Overrides from class PObject */
+  //@{
+    /**Output the string to the specified stream.
+     */
+    virtual void PrintOn(
+      ostream & strm  ///< I/O stream to output to.
+    ) const;
+
+    /**Input the string from the specified stream. This will read all
+       characters until a end of line is reached, then parsing the arguments.
+     */
+    virtual void ReadFrom(
+      istream & strm  ///< I/O stream to input from. 
     );
   //@}
 
@@ -174,16 +200,16 @@ class PArgList : public PObject
     /** Set the internal copy of the program arguments.
     */
     void SetArgs(
-      const PString & theArgStr /// A string constituting the arguments
+      const PString & theArgStr ///< A string constituting the arguments 
     );
     /** Set the internal copy of the program arguments. */
     void SetArgs(
-      int theArgc,     /// Count of argument strings in theArgv
-      char ** theArgv  /// An array of strings constituting the arguments
+      int theArgc,     ///< Count of argument strings in theArgv 
+      char ** theArgv  ///< An array of strings constituting the arguments 
     );
     /** Set the internal copy of the program arguments. */
     void SetArgs(
-      const PStringArray & theArgs /// A string array constituting the arguments
+      const PStringArray & theArgs ///< A string array constituting the arguments
     );
 
     /** Parse the arguments.
@@ -209,19 +235,13 @@ class PArgList : public PObject
        @return TRUE if there is at least one parameter after parsing.
      */
     virtual BOOL Parse(
-      const char * theArgumentSpec,
-      /** The specification string for argument options. See description for
-         details.
-       */
-      BOOL optionsBeforeParams = TRUE  /// Parse options only before parameters
+      const char * theArgumentSpec,    ///< The specification string for argument options. See description for details.
+      BOOL optionsBeforeParams = TRUE  ///< Parse options only before parameters
     );
     /** Parse the arguments. */
     virtual BOOL Parse(
-      const PString & theArgumentStr,
-      /** The specification string for argument options. See description for
-         details.
-       */
-      BOOL optionsBeforeParams = TRUE  /// Parse options only before parameters
+      const PString & theArgumentStr,  ///< The specification string for argument options. See description for details.       
+      BOOL optionsBeforeParams = TRUE  ///< Parse options only before parameters
     );
   //@}
 
@@ -233,15 +253,15 @@ class PArgList : public PObject
        @return option repeat count.
      */
     virtual PINDEX GetOptionCount(
-      char optionChar  /// Character letter code for the option
+      char optionChar        ///< Character letter code for the option 
     ) const;
     /** Get the count of option */
     virtual PINDEX GetOptionCount(
-      const char * optionStr /// String code for the option
+      const char * optionStr ///< String code for the option 
     ) const;
     /** Get the count of option */
     virtual PINDEX GetOptionCount(
-      const PString & optionName /// String code for the option
+      const PString & optionName ///< String code for the option 
     ) const;
 
     /** Get if option present.
@@ -250,15 +270,15 @@ class PArgList : public PObject
        @return TRUE if the option was present.
      */
     BOOL HasOption(
-      char optionChar  /// Character letter code for the option
+      char optionChar             ///< Character letter code for the option 
     ) const;
     /** Get if option present. */
     BOOL HasOption(
-      const char * optionStr /// String letter code for the option
+      const char * optionStr     ///< String letter code for the option 
     ) const;
     /** Get if option present. */
     BOOL HasOption(
-      const PString & optionName /// String code for the option
+      const PString & optionName ///<  String code for the option 
     ) const;
 
     /** Get option string.
@@ -270,18 +290,18 @@ class PArgList : public PObject
        @return the options associated string.
      */
     virtual PString GetOptionString(
-      char optionChar,          /// Character letter code for the option
-      const char * dflt = NULL  /// Default value of the option string
+      char optionChar,          ///< Character letter code for the option 
+      const char * dflt = NULL  ///< Default value of the option string 
     ) const;
     /** Get option string. */
     virtual PString GetOptionString(
-      const char * optionStr,   /// String letter code for the option
-      const char * dflt = NULL  /// Default value of the option string
+      const char * optionStr,   ///< String letter code for the option 
+      const char * dflt = NULL  ///<Default value of the option string 
     ) const;
     /** Get option string. */
     virtual PString GetOptionString(
-      const PString & optionName, /// String code for the option
-      const char * dflt = NULL    /// Default value of the option string
+      const PString & optionName, ///< String code for the option 
+      const char * dflt = NULL    ///< Default value of the option string 
     ) const;
 
     /** Get the argument count.
@@ -293,12 +313,21 @@ class PArgList : public PObject
      */
     PINDEX GetCount() const;
 
+    /** Get the parameters that were parsed in the argument list.
+
+       @return array of parameter strings at the specified index range.
+     */
+    PStringArray GetParameters(
+      PINDEX first = 0,
+      PINDEX last = P_MAX_INDEX
+    ) const;
+
     /** Get the parameter that was parsed in the argument list.
 
        @return parameter string at the specified index.
      */
     PString GetParameter(
-      PINDEX num   /// Number of the parameter to retrieve.
+      PINDEX num   ///< Number of the parameter to retrieve. 
     ) const;
 
     /** Get the parameter that was parsed in the argument list. The argument
@@ -307,28 +336,28 @@ class PArgList : public PObject
        @return parameter string at the specified index.
      */
     PString operator[](
-      PINDEX num   /// Number of the parameter to retrieve.
+      PINDEX num   ///< Number of the parameter to retrieve. 
     ) const;
 
     /** Shift the parameters by the specified amount. This allows the parameters
        to be parsed at the same position in the argument list "array".
      */
     void Shift(
-      int sh /// Number of parameters to shift forward through list
+      int sh ///< Number of parameters to shift forward through list 
     );
 
     /** Shift the parameters by the specified amount. This allows the parameters
        to be parsed at the same position in the argument list "array".
      */
     PArgList & operator<<(
-      int sh /// Number of parameters to shift forward through list
+      int sh ///< Number of parameters to shift forward through list 
     );
 
     /** Shift the parameters by the specified amount. This allows the parameters
        to be parsed at the same position in the argument list "array".
      */
     PArgList & operator>>(
-      int sh /// Number of parameters to shift backward through list
+      int sh ///< Number of parameters to shift backward through list 
     );
   //@}
 
@@ -339,7 +368,7 @@ class PArgList : public PObject
        message to the standard #PError# stream.
      */
     virtual void IllegalArgumentIndex(
-      PINDEX idx /// Number of the parameter that was accessed.
+      PINDEX idx ///< Number of the parameter that was accessed. 
     ) const;
 
     /** This function is called when an unknown option was specified on the
@@ -347,7 +376,7 @@ class PArgList : public PObject
        standard #PError# stream.
      */
     virtual void UnknownOption(
-      const PString & option   /// Option that was illegally placed on command line.
+      const PString & option   ///< Option that was illegally placed on command line. 
     ) const;
 
     /** This function is called when an option that requires an associated
@@ -356,7 +385,7 @@ class PArgList : public PObject
        #PError# stream.
      */
     virtual void MissingArgument(
-      const PString & option  /// Option for which the associated string was missing.
+      const PString & option  ///< Option for which the associated string was missing. 
     ) const;
   //@}
 
@@ -383,6 +412,8 @@ class PArgList : public PObject
 };
 
 
+#ifdef P_CONFIG_FILE
+
 /**This class parse command line arguments with the ability to override them
    from a PConfig file/registry.
   */
@@ -393,7 +424,7 @@ class PConfigArgs : public PArgList
   /**@name Construction */
   //@{
     PConfigArgs(
-      const PArgList & args   /// Raw argument list.
+      const PArgList & args   ///< Raw argument list.
     );
   //@}
 
@@ -405,15 +436,15 @@ class PConfigArgs : public PArgList
        @return option repeat count.
      */
     virtual PINDEX GetOptionCount(
-      char optionChar  /// Character letter code for the option
+      char optionChar  ///< Character letter code for the option
     ) const;
     /** Get the count of option */
     virtual PINDEX GetOptionCount(
-      const char * optionStr /// String code for the option
+      const char * optionStr ///< String code for the option
     ) const;
     /** Get the count of option */
     virtual PINDEX GetOptionCount(
-      const PString & optionName /// String code for the option
+      const PString & optionName ///< String code for the option
     ) const;
 
     /** Get option string.
@@ -425,18 +456,20 @@ class PConfigArgs : public PArgList
        @return the options associated string.
      */
     virtual PString GetOptionString(
-      char optionChar,          /// Character letter code for the option
-      const char * dflt = NULL  /// Default value of the option string
+      char optionChar,          ///< Character letter code for the option 
+      const char * dflt = NULL  ///< Default value of the option string 
     ) const;
+
     /** Get option string. */
     virtual PString GetOptionString(
-      const char * optionStr,   /// String letter code for the option
-      const char * dflt = NULL  /// Default value of the option string
+      const char * optionStr,   ///< String letter code for the option 
+      const char * dflt = NULL  ///< Default value of the option string 
     ) const;
+
     /** Get option string. */
     virtual PString GetOptionString(
-      const PString & optionName, /// String code for the option
-      const char * dflt = NULL    /// Default value of the option string
+      const PString & optionName, ///< String code for the option 
+      const char * dflt = NULL    ///< Default value of the option string 
     ) const;
   //@}
 
@@ -449,13 +482,13 @@ class PConfigArgs : public PArgList
        PConfig itself as this would cause the data to be saved always!
       */
     void Save(
-      const PString & optionName   /// Option name for saving.
+      const PString & optionName   ///< Option name for saving.
     );
 
     /**Set the PConfig section name for options.
       */
     void SetSectionName(
-      const PString & section /// New section name
+      const PString & section ///< New section name 
     ) { sectionName = section; }
 
     /**Get the PConfig section name for options.
@@ -466,7 +499,7 @@ class PConfigArgs : public PArgList
        The default is "no-".
       */
     void SetNegationPrefix(
-      const PString & prefix /// New prefix string
+      const PString & prefix ///< New prefix string 
     ) { negationPrefix = prefix; }
 
     /**Get the prefix for option negation.
@@ -483,6 +516,7 @@ class PConfigArgs : public PArgList
     PString negationPrefix;
 };
 
+#endif // P_CONFIG_FILE
 
 #endif
 
