@@ -27,6 +27,71 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: osutil.cxx,v $
+ * Revision 1.85  2005/11/30 12:47:42  csoutheren
+ * Removed tabs, reformatted some code, and changed tags for Doxygen
+ *
+ * Revision 1.84  2005/11/25 02:29:54  csoutheren
+ * Applied patch #1353487 by Hannes Friederich
+ * Mac OS X update
+ *
+ * Revision 1.83  2004/09/23 05:00:10  csoutheren
+ * Extra proofing against possible NULL pointers
+ *
+ * Revision 1.82  2004/04/03 23:53:10  csoutheren
+ * Added various changes to improce compatibility with the Sun Forte compiler
+ *   Thanks to Brian Cameron
+ * Added detection of readdir_r version
+ *
+ * Revision 1.81  2004/04/01 12:51:11  csoutheren
+ * Fixed problem with args to access, thanks to Borko Jandras
+ *
+ * Revision 1.80  2004/01/07 21:30:30  dsandras
+ * Applied patch from Miguel Rodriguez Perez <migrax@terra.es> to remove problematic PAssert failing in cases where it shouldn't.
+ *
+ * Revision 1.79  2003/09/17 01:18:04  csoutheren
+ * Removed recursive include file system and removed all references
+ * to deprecated coooperative threading support
+ *
+ * Revision 1.78  2003/05/01 06:08:36  robertj
+ * Fixed concurrency problem with some time functions, thanks chad@broadmind.com
+ *
+ * Revision 1.77  2003/01/24 10:21:06  robertj
+ * Fixed issues in RTEMS support, thanks Vladimir Nesic
+ *
+ * Revision 1.76  2002/11/22 10:14:07  robertj
+ * QNX port, thanks Xiaodan Tang
+ *
+ * Revision 1.75  2002/11/20 01:55:06  robertj
+ * Fixed to follow new semantics of GetPath(), first entry is volume which on
+ *   UNix machines is always an empty string. Also collapses consecutive
+ *   slashes as they are meaningless.
+ *
+ * Revision 1.74  2002/11/19 11:21:30  robertj
+ * Changed PFilePath so can be empty string, indicating illegal path.
+ * Added function to extract a path as an array of directories components.
+ *
+ * Revision 1.73  2002/10/22 07:42:52  robertj
+ * Added extra debugging for file handle and thread leak detection.
+ *
+ * Revision 1.72  2002/10/17 13:44:27  robertj
+ * Port to RTEMS, thanks Vladimir Nesic.
+ *
+ * Revision 1.71  2002/10/17 12:57:24  robertj
+ * Added ability to increase maximum file handles on a process.
+ *
+ * Revision 1.70  2002/10/10 04:43:44  robertj
+ * VxWorks port, thanks Martijn Roest
+ *
+ * Revision 1.69  2002/06/06 09:28:42  robertj
+ * Fixed problems with canonicalising directories now PINDEX is signed.
+ *
+ * Revision 1.68  2002/02/11 02:26:54  craigs
+ * Fixed problem with reading lines of length > 100 chares from text files
+ * Thanks to Ohnuma Masato
+ *
+ * Revision 1.67  2002/01/26 23:58:15  craigs
+ * Changed for GCC 3.0 compatibility, thanks to manty@manty.net
+ *
  * Revision 1.66  2001/10/11 02:20:54  robertj
  * Added IRIX support (no audio/video), thanks Andre Schulze.
  *
@@ -163,8 +228,12 @@
 
 
 #include <fcntl.h>
+#ifdef P_VXWORKS
+#include <sys/times.h>
+#else
 #include <time.h>
 #include <sys/time.h>
+#endif
 #include <ctype.h>
 
 #if defined(P_LINUX)
@@ -172,9 +241,7 @@
 #include <mntent.h>
 #include <sys/vfs.h>
 
-#define P_HAS_READDIR_R
-
-#if (__GNUC_MINOR__ < 7)
+#if (__GNUC_MINOR__ < 7 && __GNUC__ < 3)
 #include <localeinfo.h>
 #else
 #define P_USE_LANGINFO
@@ -197,7 +264,6 @@
 #include <sys/statfs.h>
 
 #elif defined(P_SOLARIS) 
-#define P_HAS_READDIR_R
 #define P_USE_LANGINFO
 #include <sys/timeb.h>
 #include <sys/statvfs.h>
@@ -216,19 +282,37 @@
 #include <stdio.h>
 #include <mntent.h>
 
+#elif defined(P_VXWORKS)
+#define P_USE_STRFTIME
+
+#elif defined(P_RTEMS)
+#define P_USE_STRFTIME
+#include <time.h>
+#include <stdio.h>
+#define random() rand()
+#define srandom(a) srand(a)
+
+#elif defined(P_QNX)
+#include <sys/dcmd_blk.h>
+#include <sys/statvfs.h>
+#define P_USE_STRFTIME
 #endif
 
 #ifdef P_USE_LANGINFO
 #include <langinfo.h>
 #endif
 
-#define	LINE_SIZE_STEP	100
+#define  LINE_SIZE_STEP  100
 
-#define	DEFAULT_FILE_MODE	(S_IRUSR|S_IWUSR|S_IROTH|S_IRGRP)
+#define  DEFAULT_FILE_MODE  (S_IRUSR|S_IWUSR|S_IROTH|S_IRGRP)
 
 #if !P_USE_INLINES
-#include <ptlib/osutil.inl>
-#include <ptlib/ptlib.inl>
+#include "ptlib/osutil.inl"
+#ifdef _WIN32
+#include "ptlib/win32/ptlib/ptlib.inl"
+#else
+#include "ptlib/unix/ptlib/ptlib.inl"
+#endif
 #endif
 
 #ifdef P_SUN4
@@ -245,22 +329,48 @@ static char *tzname[2] = { "STD", "DST" };
 #define new PNEW
 
 
+int PX_NewHandle(const char * clsName, int fd)
+{
+  if (fd < 0)
+    return fd;
+
+  static int lowWaterMark = INT_MAX;
+  static int highWaterMark = 0;
+  if (fd > highWaterMark) {
+    highWaterMark = fd;
+    lowWaterMark = fd;
+
+    int maxHandles = PProcess::Current().GetMaxHandles();
+    if (fd < (maxHandles-maxHandles/20))
+      PTRACE(4, "PWLib\tFile handle high water mark set: " << fd << ' ' << clsName);
+    else
+      PTRACE(1, "PWLib\tFile handle high water mark within 5% of maximum: " << fd << ' ' << clsName);
+  }
+
+  if (fd < lowWaterMark) {
+    lowWaterMark = fd;
+    PTRACE(4, "PWLib\tFile handle low water mark set: " << fd << ' ' << clsName);
+  }
+
+  return fd;
+}
+
+
 static PString CanonicaliseDirectory (const PString & path)
 
 {
   PString canonical_path;
-  PString slash("/");
 
-  // if the path does not start with a slash, then the current directory
-  // must be prepended
-  if (path.IsEmpty() || path[0] != '/')  {
+  if (path[0] == '/')
+    canonical_path = '/';
+  else {
     char *p = getcwd(canonical_path.GetPointer(P_MAX_PATH), P_MAX_PATH);
     PAssertOS (p != NULL);
-  }
 
-  // if the path doesn't end in a slash, add one
-  if (canonical_path[canonical_path.GetLength()-1] != '/')
-    canonical_path += slash;
+    // if the path doesn't end in a slash, add one
+    if (canonical_path[canonical_path.GetLength()-1] != '/')
+      canonical_path += '/';
+  }
 
   const char * ptr = path;
   const char * end;
@@ -289,7 +399,7 @@ static PString CanonicaliseDirectory (const PString & path)
     } else if (element == "." || element == "") {
     } else {
       canonical_path += element;
-      canonical_path += slash;
+      canonical_path += '/';
     }
     ptr = end;
   }
@@ -297,9 +407,12 @@ static PString CanonicaliseDirectory (const PString & path)
   return canonical_path;
 }
 
-static PString CanonicaliseFilename(const PString & filename)
 
+static PString CanonicaliseFilename(const PString & filename)
 {
+  if (filename.IsEmpty())
+    return filename;
+
   PINDEX p;
   PString dirname;
 
@@ -318,8 +431,10 @@ static PString CanonicaliseFilename(const PString & filename)
 PInt64 PString::AsInt64(unsigned base) const
 {
   char * dummy;
-#if defined(P_SOLARIS) || defined(__BEOS__) || defined (P_AIX) || defined(P_IRIX)
+#if defined(P_SOLARIS) || defined(__BEOS__) || defined (P_AIX) || defined(P_IRIX) || defined (P_QNX)
   return strtoll(theArray, &dummy, base);
+#elif defined(P_VXWORKS) || defined(P_RTEMS)
+  return strtol(theArray, &dummy, base);
 #else
   return strtoq(theArray, &dummy, base);
 #endif
@@ -328,30 +443,16 @@ PInt64 PString::AsInt64(unsigned base) const
 PUInt64 PString::AsUnsigned64(unsigned base) const
 {
   char * dummy;
-#if defined(P_SOLARIS) || defined(__BEOS__) || defined (P_AIX) || defined (P_IRIX)
+#if defined(P_SOLARIS) || defined(__BEOS__) || defined (P_AIX) || defined (P_IRIX) || defined (P_QNX)
   return strtoull(theArray, &dummy, base);
+#elif defined(P_VXWORKS) || defined(P_RTEMS)
+  return strtoul(theArray, &dummy, base);
 #else
   return strtouq(theArray, &dummy, base);
 #endif
 }
 
 
-///////////////////////////////////////////////////////////////////////////////
-//
-// time interval
-
-struct timeval * PTimeInterval::AsTimeVal(struct timeval & buffer) const
-{
-  if (*this == PMaxTimeInterval)
-    return NULL;
-
-  buffer.tv_usec = (milliseconds % 1000) * 1000;
-  buffer.tv_sec  =  milliseconds / 1000;
-  return &buffer;
-}
-
-
-
 ///////////////////////////////////////////////////////////////////////////////
 //
 // timer
@@ -360,9 +461,15 @@ struct timeval * PTimeInterval::AsTimeVal(struct timeval & buffer) const
 PTimeInterval PTimer::Tick()
 
 {
+#ifdef P_VXWORKS
+  struct timespec ts;
+  clock_gettime(0,&ts);
+  return (int)(ts.tv_sec*10000) + ts.tv_nsec/100000L;
+#else
   struct timeval tv;
   ::gettimeofday (&tv, NULL);
   return (PInt64)(tv.tv_sec) * 1000 + tv.tv_usec/1000L;
+#endif // P_VXWORKS
 }
 
 
@@ -403,7 +510,6 @@ void PDirectory::Close()
 }
 
 void PDirectory::Construct ()
-
 {
   directory   = NULL;
   entryBuffer = NULL;
@@ -443,20 +549,27 @@ BOOL PDirectory::Next()
     do {
       struct dirent * entryPtr;
       entryBuffer->d_name[0] = '\0';
-#ifdef P_HAS_READDIR_R
+#if P_HAS_POSIX_READDIR_R == 3
       if (::readdir_r(directory, entryBuffer, &entryPtr) != 0)
         return FALSE;
       if (entryPtr != entryBuffer)
         return FALSE;
+#elif P_HAS_POSIX_READDIR_R == 2
+      entryPtr = ::readdir_r(directory, entryBuffer);
+      if (entryPtr == NULL)
+        return FALSE;
 #else
-      if ((entryPtr = readdir(directory)) == NULL)
+      if ((entryPtr = ::readdir(directory)) == NULL)
         return FALSE;
       *entryBuffer = *entryPtr;
       strcpy(entryBuffer->d_name, entryPtr->d_name);
 #endif
     } while (strcmp(entryBuffer->d_name, "." ) == 0 || strcmp(entryBuffer->d_name, "..") == 0);
 
-    PAssert(PFile::GetInfo(*this+entryBuffer->d_name, *entryInfo), POperatingSystemError);
+    /* Ignore this file if we can't get info about it */
+    if (PFile::GetInfo(*this+entryBuffer->d_name, *entryInfo) == 0)
+      continue;
+    
     if (scanMask == PFileInfo::AllPermissions)
       return TRUE;
   } while ((entryInfo->type & scanMask) == 0);
@@ -492,6 +605,9 @@ PString PDirectory::GetEntryName() const
 
 BOOL PDirectory::GetInfo(PFileInfo & info) const
 {
+  if (entryInfo == NULL)
+    return FALSE;
+
   info = *entryInfo;
   return TRUE;
 }
@@ -514,7 +630,11 @@ BOOL PDirectory::Create(const PString & p, int perm)
   PString str = p;
   if (p[last] == '/')
     str = p.Left(last);
+#ifdef P_VXWORKS
+  return mkdir(str) == 0;
+#else    
   return mkdir(str, perm) == 0;
+#endif
 }
 
 BOOL PDirectory::Remove(const PString & p)
@@ -528,6 +648,18 @@ PString PDirectory::GetVolume() const
 {
   PString volume;
 
+#if defined(P_QNX)
+  int fd;
+  char mounton[257];
+
+  if ((fd = open(operator+("."), O_RDONLY)) != -1) {
+  mounton[256] = 0;
+  devctl(fd, DCMD_FSYS_MOUNTED_ON, mounton, 256, 0);
+  close(fd);
+  volume = strdup(mounton);
+  } 
+
+#else
   struct stat status;
   if (stat(operator+("."), &status) != -1) {
     dev_t my_dev = status.st_dev;
@@ -583,13 +715,17 @@ PString PDirectory::GetVolume() const
     }
     endfsent();
 
-#else
+#elif defined (P_VXWORKS)
 
+  PAssertAlways("Get Volume - not implemented for VxWorks");
+  return PString::Empty();
+
+#else
 #warning Platform requires implemetation of GetVolume()
 
 #endif
-
   }
+#endif
 
   return volume;
 }
@@ -608,7 +744,7 @@ BOOL PDirectory::GetVolumeSpace(PInt64 & total, PInt64 & free, DWORD & clusterSi
   free = fs.f_bavail*(PInt64)fs.f_bsize;
   return TRUE;
 
-#elif defined(P_AIX)
+#elif defined(P_AIX) || defined(P_VXWORKS)
 
   struct statfs fs;
   if (statfs((char *) ((const char *)operator+(".") ), &fs) == -1)
@@ -643,6 +779,18 @@ BOOL PDirectory::GetVolumeSpace(PInt64 & total, PInt64 & free, DWORD & clusterSi
   free = fs.f_bfree*(PInt64)fs.f_bsize;
   return TRUE;
 
+#elif defined(P_QNX)
+
+  struct statvfs fs;
+
+  if (statvfs(operator+("."), &fs) == -1)
+    return FALSE;
+
+  clusterSize = fs.f_bsize;
+  total = fs.f_blocks*(PInt64)fs.f_bsize;
+  free = fs.f_bavail*(PInt64)fs.f_bsize;
+  return TRUE;
+
 #else
 
 #warning Platform requires implemetation of GetVolumeSpace()
@@ -659,6 +807,29 @@ PDirectory PDirectory::GetParent() const
   return *this + "..";
 }
 
+PStringArray PDirectory::GetPath() const
+{
+  PStringArray path;
+
+  if (IsEmpty())
+    return path;
+
+  PStringArray tokens = Tokenise("/");
+
+  path.SetSize(tokens.GetSize()+1);
+
+  PINDEX count = 1; // First path field is volume name, empty under unix
+  for (PINDEX i = 0; i < tokens.GetSize(); i++) {
+    if (!tokens[i])
+     path[count++] = tokens[i];
+  }
+
+  path.SetSize(count);
+
+  return path;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 // PFile
@@ -687,11 +858,25 @@ BOOL PFile::Open(OpenMode mode, int opt)
   if (path.IsEmpty()) {
     char templateStr[3+6+1];
     strcpy(templateStr, "PWL");
+#ifndef P_VXWORKS
+#ifdef P_RTEMS
+    _reent _reent_data;
+    memset(&_reent_data, 0, sizeof(_reent_data));
+    os_handle = _mkstemp_r(&_reent_data, templateStr);
+#else
     os_handle = mkstemp(templateStr);
+#endif // P_RTEMS
     if (!ConvertOSError(os_handle))
       return FALSE;
 
   } else {
+#else
+    static int number = 0;
+    sprintf(templateStr+3, "%06d", number++);
+    path = templateStr;
+  }
+  {
+#endif // !P_VXWORKS
     int oflags = 0;
     switch (mode) {
       case ReadOnly :
@@ -721,11 +906,15 @@ BOOL PFile::Open(OpenMode mode, int opt)
       oflags |= O_TRUNC;
 
 
-    if (!ConvertOSError(os_handle = ::open(path, oflags, DEFAULT_FILE_MODE)))
+    if (!ConvertOSError(os_handle = PX_NewHandle(GetClass(), ::open(path, oflags, DEFAULT_FILE_MODE))))
       return FALSE;
   }
 
+#ifndef P_VXWORKS
   return ConvertOSError(::fcntl(os_handle, F_SETFD, 1));
+#else
+  return TRUE;
+#endif
 }
 
 
@@ -769,30 +958,58 @@ BOOL PFile::Move(const PFilePath & oldname, const PFilePath & newname, BOOL forc
   if (force && errno == EEXIST)
     if (Remove(to, TRUE))
       if (rename(from, to) == 0)
-	return TRUE;
+  return TRUE;
 
   return FALSE;
 }
 
 
+BOOL PFile::Exists(const PFilePath & name)
+{ 
+#ifdef P_VXWORKS
+  // access function not defined for VxWorks
+  // as workaround, open the file in read-only mode
+  // if it succeeds, the file exists
+  PFile file(name, ReadOnly, MustExist);
+  BOOL exists = file.IsOpen();
+  if(exists == TRUE)
+    file.Close();
+  return exists;
+#else
+  return access(name, 0) == 0; 
+#endif // P_VXWORKS
+}
+
+
 BOOL PFile::Access(const PFilePath & name, OpenMode mode)
 {
+#ifdef P_VXWORKS
+  // access function not defined for VxWorks
+  // as workaround, open the file in specified mode
+  // if it succeeds, the access is allowed
+  PFile file(name, mode, ModeDefault);
+  BOOL access = file.IsOpen();
+  if(access == TRUE)
+    file.Close();
+  return access;
+#else  
   int accmode;
 
   switch (mode) {
     case ReadOnly :
-      accmode = 2;
+      accmode = R_OK;
       break;
 
     case WriteOnly :
-      accmode = 4;
+      accmode = W_OK;
       break;
 
     default :
-      accmode = 6;
+      accmode = R_OK | W_OK;
   }
 
   return access(name, accmode) == 0;
+#endif // P_VXWORKS
 }
 
 
@@ -801,9 +1018,14 @@ BOOL PFile::GetInfo(const PFilePath & name, PFileInfo & status)
   status.type = PFileInfo::UnknownFileType;
 
   struct stat s;
+#ifdef P_VXWORKS
+  if (stat(name, &s) != OK)
+#else  
   if (lstat(name, &s) != 0)
+#endif // P_VXWORKS
     return FALSE;
 
+#ifndef P_VXWORKS
   if (S_ISLNK(s.st_mode)) {
     status.type = PFileInfo::SymbolicLink;
     if (stat(name, &s) != 0) {
@@ -815,7 +1037,9 @@ BOOL PFile::GetInfo(const PFilePath & name, PFileInfo & status)
       return TRUE;
     }
   } 
-  else if (S_ISREG(s.st_mode))
+  else 
+#endif // !P_VXWORKS
+  if (S_ISREG(s.st_mode))
     status.type = PFileInfo::RegularFile;
   else if (S_ISDIR(s.st_mode))
     status.type = PFileInfo::SubDirectory;
@@ -825,10 +1049,10 @@ BOOL PFile::GetInfo(const PFilePath & name, PFileInfo & status)
     status.type = PFileInfo::CharDevice;
   else if (S_ISBLK(s.st_mode))
     status.type = PFileInfo::BlockDevice;
-#ifndef __BEOS__
+#if !defined(__BEOS__) && !defined(P_VXWORKS)
   else if (S_ISSOCK(s.st_mode))
     status.type = PFileInfo::SocketDevice;
-#endif // !__BEOS__
+#endif // !__BEOS__ || !P_VXWORKS
 
   status.created     = s.st_ctime;
   status.modified    = s.st_mtime;
@@ -869,7 +1093,15 @@ BOOL PFile::SetPermissions(const PFilePath & name, int permissions)
   if (permissions & PFileInfo::UserRead)
     mode |= S_IRUSR;
 
+#ifdef P_VXWORKS
+  PFile file(name, ReadOnly, MustExist);
+  if (file.IsOpen())
+    return (::ioctl(file.GetHandle(), FIOATTRIBSET, mode) >= 0);
+
+  return FALSE;
+#else  
   return chmod ((const char *)name, mode) == 0;
+#endif // P_VXWORKS
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -895,7 +1127,7 @@ BOOL PTextFile::ReadLine (PString & line)
 
   while (1) {
     len += LINE_SIZE_STEP;
-    ptr = base = line.GetPointer(len);
+    ptr = base = line.GetPointer(len) + len - LINE_SIZE_STEP;
     while ((ptr - base) < LINE_SIZE_STEP-1) {
       if ((ch = ReadChar()) < 0) {
         ConvertOSError(errno);
@@ -936,6 +1168,14 @@ PFilePath::PFilePath(const char * prefix, const char * dir)
   if (dir == NULL) 
     s = PDirectory("/tmp");
 
+#ifdef P_VXWORKS
+  int number = 0;
+  for (;;) {
+    *this = s + prefix + psprintf("%06x", number++);
+    if (!PFile::Exists(*this))
+      break;
+  }
+#else
   PString p;
   srandom(getpid());
   for (;;) {
@@ -943,6 +1183,7 @@ PFilePath::PFilePath(const char * prefix, const char * dir)
     if (!PFile::Exists(*this))
       break;
   }
+#endif // P_VXWORKS
 }
 
 
@@ -1065,7 +1306,12 @@ BOOL PConsoleChannel::Open(ConsoleType type)
 
 PString PConsoleChannel::GetName() const
 {
+#ifdef P_VXWORKS
+  PAssertAlways("PConsoleChannel::GetName - Not implemented for VxWorks");
+  return PString("Not Implemented");
+#else
   return ttyname(os_handle);
+#endif // P_VXWORKS
 }
 
 
@@ -1083,10 +1329,17 @@ BOOL PConsoleChannel::Close()
 
 PTime::PTime()
 {
+#ifdef P_VXWORKS
+  struct timespec ts;
+  clock_gettime(0,&ts);
+  theTime = ts.tv_sec;
+  microseconds = ts.tv_sec*10000 + ts.tv_nsec/100000L;
+#else
   struct timeval tv;
   gettimeofday(&tv, NULL);
   theTime = tv.tv_sec;
   microseconds = tv.tv_usec;
+#endif // P_VXWORKS
 }
 
 
@@ -1330,7 +1583,8 @@ PString PTime::GetMonthName(PTime::Months month, NameType type)
 BOOL PTime::IsDaylightSavings()
 {
   time_t theTime = ::time(NULL);
-  return ::localtime(&theTime)->tm_isdst != 0;
+  struct tm ts;
+  return os_localtime(&theTime, &ts)->tm_isdst != 0;
 }
 
 int PTime::GetTimeZone(PTime::TimeZoneType type) 
@@ -1341,10 +1595,11 @@ int PTime::GetTimeZone(PTime::TimeZoneType type)
     return tz;
   else
     return tz + ::daylight*60;
-#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined(__BEOS__)
+#elif defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined(__BEOS__) || defined(P_QNX)
   time_t t;
   time(&t);
-  struct tm  * tm = localtime(&t);
+  struct tm ts;
+  struct tm * tm = os_localtime(&t, &ts);
   int tz = tm->tm_gmtoff/60;
   if (type == StandardTime && tm->tm_isdst)
     return tz-60;
@@ -1366,7 +1621,7 @@ int PTime::GetTimeZone(PTime::TimeZoneType type)
 
 PString PTime::GetTimeZoneString(PTime::TimeZoneType type) 
 {
-#if defined(P_LINUX) || defined(P_SUN4) || defined(P_SOLARIS) || defined (P_AIX) || defined(P_IRIX)
+#if defined(P_LINUX) || defined(P_SUN4) || defined(P_SOLARIS) || defined (P_AIX) || defined(P_IRIX) || defined(P_QNX)
   const char * str = (type == StandardTime) ? ::tzname[0] : ::tzname[1]; 
   if (str != NULL)
     return str;
@@ -1389,12 +1644,7 @@ PString PTime::GetTimeZoneString(PTime::TimeZoneType type)
 #ifdef P_PTHREADS
 struct tm * PTime::os_localtime(const time_t * clock, struct tm * ts)
 {
-#ifdef P_MACOSX 
-#warning LOCALTIME_R NOT DEFINED
-  return ::localtime(clock);
-#else
   return ::localtime_r(clock, ts);
-#endif
 }
 #else
 struct tm * PTime::os_localtime(const time_t * clock, struct tm *)
@@ -1406,12 +1656,7 @@ struct tm * PTime::os_localtime(const time_t * clock, struct tm *)
 #ifdef P_PTHREADS
 struct tm * PTime::os_gmtime(const time_t * clock, struct tm * ts)
 {
-#ifdef P_MACOSX
-#warning GMTIME_R NOT DEFINED
-  return ::gmtime(clock);
-#else
   return ::gmtime_r(clock, ts);
-#endif
 }
 #else
 struct tm * PTime::os_gmtime(const time_t * clock, struct tm *)
